/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

//#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_

#include "qt_gui_datadir_fallback.h"

#include <QtGui/QApplication>
#include <QtGui/QToolBox>
#include <QtGui/QGridLayout>

#include "qt_gui_elements.h"
#include "q_image_widget.h"

#include "gui_general.h"
#include "gui_video.h"
#include "gui_audio.h"
#include "gui_control.h"

#include "banner.xpm"

namespace OrxGui
{
  QtGuiDataDirFallback::QtGuiDataDirFallback(int argc, char** argv)
  : QApplication(argc, argv)
  {

    this->mainWindow = new QMainWindow();
    this->mainWindow->setMinimumSize(500, 200);

    QGroupBox* groupBox = new QGroupBox(this->mainWindow);
    QGridLayout* mainLayout = new QGridLayout(groupBox);
    {
      QImageWidget* bannerWidget = new QImageWidget();
      QImage bannerImage(banner_xpm);
      bannerImage.save("test.bmp", "bmp");
      bannerWidget->setImage(bannerImage);
      mainLayout->addWidget(bannerWidget, 0,0, 3, 1);

      QLabel* label = new QLabel();
      label->setText("ORXONOX needs its DataDirectory to be configured.\n Just Select the ORXONOX-dataDirectory with the browse-button, and start the Game\n");

      mainLayout->addWidget(label, 0,1);

      mainLayout->addWidget(new GuiGeneral(this), 1, 1, 1, 3);

      QPushButton* start = new QPushButton("start");
      connect(start, SIGNAL(released()), this, SLOT(startApp()));
      mainLayout->addWidget(start, 2,2);

      QPushButton* quit = new QPushButton("quit");
      connect(quit, SIGNAL(released()), this, SLOT(quitApp()));
      mainLayout->addWidget(quit, 2,3);

    }

    connect(this->mainWindow, SIGNAL(destroyed()), this, SLOT(quitApp()));
    this->mainWindow->setCentralWidget(groupBox);
    this->mainWindow->show();

    this->loadAll();
    this->exec();
    this->saveAll();

  }

  QtGuiDataDirFallback::~QtGuiDataDirFallback()
  {
    delete this->mainWindow;
  }

  void QtGuiDataDirFallback::startGui()
  {
  }

  void QtGuiDataDirFallback::stopGui()
  {}

  void QtGuiDataDirFallback::suspend()
  {}

    //! Update the Gui.
  void QtGuiDataDirFallback::update()
  {
    this->processEvents();
  }


  void QtGuiDataDirFallback::quitApp()
  {
    Gui::quitEvent();
    this->quit();
  }

  void QtGuiDataDirFallback::startApp()
  {
    Gui::startEvent();
    this->quit();
  }

}
