/*!
 * @file resource_sound_buffer.h
 * @brief Contains the ResourceSoundBuffer class, that handles the Resource-specific loading part of the SoundBuffer.
 *
 */

#ifndef _RESOURCE_SOUND_BUFFER_H
#define _RESOURCE_SOUND_BUFFER_H

#include "util/loading/resource.h"
#include "sound_buffer.h"

namespace OrxSound
{
  class ResourceSoundBuffer : public SoundBuffer, public Resources::Resource
  {
    ObjectListDeclaration(ResourceSoundBuffer);
  public:
    ResourceSoundBuffer(const std::string& soundName,
			const Resources::KeepLevel& keepLevel = Resources::KeepLevel());
    static ResourceSoundBuffer createFromString(const std::string& loadString, const Resources::KeepLevel& keepLevel = Resources::KeepLevel())
    { return ResourceSoundBuffer(loadString, keepLevel); };

  private:
  class SoundBufferResourcePointer : public Resources::StorePointer
    {
    public:
      SoundBufferResourcePointer(const std::string& loadString, const Resources::KeepLevel& keepLevel, const SoundBufferData::Pointer& data);
      inline const SoundBufferData::Pointer& ptr() const { return pointer; }
      virtual bool last() const { return pointer.count() == 1; }
    private:
      SoundBufferData::Pointer pointer;
    };

  private:
    static Resources::tType<ResourceSoundBuffer> type;
  };
}


#endif /* _RESOURCE_SOUND_BUFFER_H */
